using System;
using System.IO;              // for StreamCommon.Writer I/O to a file

namespace MyNamespace
{
	/// <summary>
	///		Calculation routines for EasterCS program
	///
	///		EasterCS - What date does Easter Sunday come on in a given year?
	///		Developed in QuickBASIC 4.50, now in Visual C# .Net
	///		COPYRIGHT (c) 1981-2008 By author: Harry J Smith, Saratoga, CA
	///
	/// ---------------- *** LEGAL NOTICE: *** ------------------
	///
	///	This program is free software; you can redistribute it and/or modify
	///	it under the terms of the GNU General Public License (GPL) as published
	///	by the Free Software Foundation. cf. the file COPYING.txt.
	///
	/// ------------ *** end of legal notice *** ------------------
	/// </summary>

	public class EasterCS
	{
		public EasterCS(int iCurrYear, int iStartYear, int iEndYear)
		// EasterCS constructor logic
		{
			currYear = iCurrYear;
			startYear = iStartYear;
			endYear = iEndYear;
			maxYear = StartupForm.maxYear;
			label = "";
		}
		public static void Easter(out int wGDay, out int wGMonth, int year)
		// The Date of Western Easter
		{
			int a;	// Intermediate integer variables
			int b;
			int c;
			int d;
			int e;
			int g;
			int h;
			int j;
			int k;
			int l;
			int m;
			int n;		// Month, 3 or 4
			int p;		// Day, 1 through 31
			a = year % 19;
			b = year / 100;
			c = year % 100;
			d = b / 4;
			e = b % 4;
			g = (8 * b + 13) / 25;
			h = (19 * a + b - d - g + 15) % 30;
			m = (a + 11 * h) / 319;
			j = c / 4;
			k = c % 4;
			l = (2 * e + 2 * j - k - h + m + 32) % 7;
			n = (h - m + l + 90) / 25;
			p = (h - m + l + n + 19) % 32;
			wGMonth = n;
			wGDay = p;
		} // Easter

		public static void EasterO(out int oJDay, out int oJMonth, out int oGDay, out int oGMonth, int year)
		// The Date of Orthodox Easter:
		// This simple algorithm is listed in the Calendar FAQ by Claus
		// Tondering. It is based on Oudin's algorithm.
		// 
		// Copyright and disclaimer
		// ------------------------
		// This document is Copyright (C) 1996 by Claus Tondering.
		// E-mail: c-t@pip.dknet.dk.
		// The document may be freely distributed, provided this
		// copyright notice is included and no money is charged for
		// the document.
		// 
		// See:
		// http://www.assa.org.au/edm.html
		// http://www.smart.net/~mmontes/ortheast.html
		// http://www.tondering.dk/claus/calendar.html
		{
			int g;		// Golden Number-1
			int i;		// number of days from 21 March to the Paschal full moon
			int j;		// weekday for the Paschal full moon (0=Sunday, 1=Monday, etc.)
			int l;		// number of days from 21 March to the Sunday on or before
						// the Pascal full moon (a number between -6 and 28)
			int rC = 0;
			// The number rC ranges from 1 to 35 which corresponds to March 22 to
			// April 25 in the Julian Calendar (currently April 4 to May 8 on the
			// Gregorian). The Julian Calendar is now 13 days behind the Gregorian,
			// and will be until March 1, 2100 when it will be 14 days behind the
			// Gregorian Calendar.
			g = year % 19;
			i = (19 * g + 15) % 30;
			j = (year + year / 4 + i) % 7;
			l = i - j;
			oJMonth = 3 + (l + 40) / 44;
			oJDay = l + 28 - 31 * (oJMonth / 4);
			if (oJMonth == 3) rC = oJDay - 21;
			if (oJMonth == 4) rC = oJDay + 10;
			oGDay = rC + 21;
			oGDay += year / 100 - year / 400 - 2;
			if (oGDay <= 31)
				oGMonth = 3;
			else if (oGDay <= 61)
			{
				oGDay -= 31;
				oGMonth = 4;
			}
			else
			{
				oGDay -= 61;
				oGMonth = 5;
			}
		}
		// EasterO

		public static void DisplayOneYear(int year)	// like EasterQB.QB
		// Dates of Western and Orthodox Easter for selected year
		// Example: Year: 2009
		// Easter Sunday: 2009  4/12 Western  4/19 Orthodox  (April  6 Julian date)
		{
			int oGDay;		// Day, 1 through 31 Orthodox Gregorian date
			int oGMonth;	// Month, 3, 4 or 5 Orthodox Gregorian date
			int oJDay;		// Day, 1 through 31 Orthodox Julian date
			int oJMonth;	// Month, 3 or 4 Orthodox Julian date
			int wGDay;		// Day, 1 through 31 Western Gregorian date
			int wGMonth;	// Month, 3 or 4 Western Gregorian date
			Easter(out wGDay, out wGMonth, year);
			string sp = "";
			if (wGDay < 10) sp = " ";
			Common.Write("Easter Sunday: " + year.ToString("0000") + "  " + wGMonth + "/" + sp + wGDay + " Western  ");
			EasterO(out oJDay, out oJMonth, out oGDay, out oGMonth, year);
			sp = "";
			if (oGDay < 10) sp = " ";
			if (wGDay == oGDay && wGMonth == oGMonth)
				Common.Write("Same");
			else
				if (oGDay > 31)
					Common.Write(">May");
				else
					Common.Write(oGMonth + "/" + sp + oGDay);
			Common.Write(" Orthodox  (");
			if (oJMonth == 3) Common.Write("March");
			if (oJMonth == 4) Common.Write("April");
			sp = "";
			if (oJDay < 10) sp = " ";
			Common.Write(" " + sp + oJDay + " Julian date)");
			Common.WriteLn();
		} // DisplayOneYear

		public static void WOByYear()	// like EasterB.QB
		// Dates of Western and Orthodox Easter, ordered by year, StartYear - EndYear
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int times = 0;					// Total times tabulated
			Common.WriteDLn("Gregorian Calendar Easter dates for years " + startYear + " - " + endYear);
			for (int year = startYear; year <= endYear; year++)
			{
				DisplayOneYear(year);
				times++;
				Common.MuInterrupt();
				if (Common.abortCalc)
				{
					Common.WriteLn("Manually aborted before list finished");
					break;
				}
			}
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		}

		public static void WByDate()	// like EasterD.QB and EasterI.QB
		// Dates of Western Easter, ordered by date, StartYear - EndYear 
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int day;							// Day tabulating
			int i;								// Index
			int times;							// Total times tabulated
			int wGDay;							// Day, 1 through 31
			int wGMonth;						// Month, 3 or 4
			int year;							// Year to compute for
			int[] tA3 = new int[32];			// Times array for March
			int[] tA4 = new int[32];			// Times array for April
			int[,] yA3 = new int[32, 387000];	// Year array for March
			int[,] yA4 = new int[32, 387000];	// Year array for April
			string sp;
			times = 0;
			Common.WriteDLn("Gregorian Calendar");
			Common.WriteLn(" Date   Years when Western Easter falls on a given date for years " + startYear + " - " + endYear);
			for (day = 1; day <= 31; day++)
			{
				tA3[day] = 0;
				tA4[day] = 0;
			}
			for (year = startYear; year <= endYear; year++)
			{
				Easter(out wGDay, out wGMonth, year);
				if (wGMonth == 3)
				{
					tA3[wGDay] = tA3[wGDay] + 1;
					yA3[wGDay, tA3[wGDay]] = year;
				}
				else
				{
					tA4[wGDay] = tA4[wGDay] + 1;
					yA4[wGDay, tA4[wGDay]] = year;
				}
			}
			int startD = 0;
			int startM = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] != 0)
				{
					startD = day;
					startM = 3;
					break;
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 30; day++)
				{
					if (tA4[day] != 0)
					{
						startD = day;
						startM = 4;
						break;
					}
				}
			}
			startD--;
			if (startD < 1)
			{
				startM--;
				startD = 31;
			}
			int stopD = 0;
			int stopM = 0;
			for (day = 30; day >= 1; day--)
			{
				if (tA4[day] != 0)
				{
					stopD = day;
					stopM = 4;
					break;
				}
			}
			if (stopM == 0)
			{
				for (day = 31; day >= 1; day--)
				{
					if (tA3[day] != 0)
					{
						stopD = day;
						stopM = 3;
						break;
					}
				}
			}
			stopD++;
			if (stopD > 31)
			{
				stopM++;
				stopD = 1;
			}
			int start;
			int stop;
			if (startM == 3)
			{
				stop = 31;
				if (stopM == 3)
					stop = stopD;
				for (day = startD; day <= stop; day++)
				{
					Common.Write(" 3/" + day + "  ");
					if (tA3[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA3[day]; i++)
					{
						Common.Write(" " + yA3[day, i].ToString("0000"));
						times++;
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
					}
					Common.WriteLn();
					if (Common.abortCalc)
						break;
				}
			}
			if (stopM == 4)
			{
				start = 1;
				if (startM == 4)
					start = startD;
				for (day = start; day <= stopD; day++)
				{
					if (Common.abortCalc)
						break;
					sp = "";
					if (day < 10) sp = " ";
					Common.Write(" 4/" + sp + day + "  ");
					if (tA4[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA4[day]; i++)
					{
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
						Common.Write(" " + yA4[day, i].ToString("0000"));
						times++;
					}
					Common.WriteLn();
				}
			}
			if (Common.abortCalc)
				Common.WriteLn("Manually aborted before list finished");
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		} // WByDate

		public static void WFreq()	// like EasterE.QB and EasterF.QB
		// Frequency of Dates of Western Easter, StartYear - EndYear
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int day;					// Day tabulating
			int month;					// Month tabulating
			int span;					// NJumber of years to include
			int times;					// Total times tabulated
			int wGDay;					// Day, 1 through 31
			int wGMonth;				// Month, 3 or 4
			int year;					// Year to compute for
			Single p;					// Percent
			times = 0;
			span = endYear - startYear + 1;
			label = "Gregorian Calendar, Frequency of Dates of Western Easter " + startYear + " - " + endYear;
			Common.WriteDLn(label);
			for (day = 1; day <= 31; day++)
			{
				tA3[day] = 0;
				tA4[day] = 0;
			}
			for (year = startYear; year <= endYear; year++)
			{
				Easter(out wGDay, out wGMonth, year);
				if (wGMonth == 3) tA3[wGDay]++;
				if (wGMonth == 4) tA4[wGDay]++;
			}
			int startM = 0;
			int startD = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] != 0)
				{
					startD = day;
					startM = 3;
					break;
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 30; day++)
				{
					if (tA4[day] != 0)
					{
						startD = day;
						startM = 4;
						break;
					}
				}
			}
			startD--;
			if (startD < 1)
			{
				startM--;
				startD = 31;
			}
			int stopM = 0;
			int stopD = 0;
			for (day = 30; day >= 1; day--)
			{
				if (tA4[day] != 0)
				{
					stopD = day;
					stopM = 4;
					break;
				}
			}
			if (stopM == 0)
			{
				for (day = 31; day >= 1; day--)
				{
					if (tA3[day] != 0)
					{
						stopD = day;
						stopM = 3;
						break;
					}
				}
			}
			stopD++;
			if (stopD > 31)
			{
				stopM++;
				stopD = 1;
			}
			int maxTimes = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] > maxTimes)
					maxTimes = tA3[day];
				if (tA4[day] > maxTimes)
					maxTimes = tA4[day];
			}
			string dayTimes = maxTimes.ToString();
			int length = dayTimes.Length;
			int start;
			int stop;
			month = 3;
			if (startM == 3)
			{
				stop = 31;
				if (stopM == 3)
					stop = stopD;
				for (day = startD; day <= stop; day++)
				{
					p = 100f * tA3[day] / span;
					dayTimes = Format(tA3[day], length);
					Common.Write("Easter Sunday: " + month.ToString() + "/" + day + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + " %");
					times += tA3[day];
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
				}
			}
			month = 4;
			if (stopM == 4)
			{
				start = 1;
				if (startM == 4)
					start = startD;
				for (day = start; day <= stopD; day++)
				{
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
					p = 100f * tA4[day] / span;
					dayTimes = Format(tA4[day], length);
					Common.Write("Easter Sunday: " + month.ToString() + "/" + Format(day, 2) + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + " %");
					times += tA4[day];
				}
			}
			if (Common.abortCalc)
			{
				Common.WriteLn("Manually aborted before list finished");
				label = "";
			}
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		} // WFreq

		public static void OByGDate()	// like EasterG.QB
		// Dates of Orthodox Easter, ordered by date, given in the Gregorian Calendar, StartYear - EndYear
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int day;						// Day tabulating
			int i;							// Index
			int oGDay;						// Day, 1 through 31 Orthodox Gregorian date, May 32 for overflow
			int oGMonth;					// Month, 3, 4 or 5 Orthodox Gregorian date
			int oJDay;						// Day, 1 through 31 Orthodox Julian date
			int oJMonth;					// Month, 3 or 4 Orthodox Julian date
			int times;						// Total times tabulated
			int year;						// Year to compute for
			int[] tA3 = new int[32];		// Times array for March
			int[] tA4 = new int[32];		// Times array for April
			int[] tA5 = new int[33];		// Times array for May
			int[,] yA3 = new int[32, 140];	// Year array for March
			int[,] yA4 = new int[32, 140];	// Year array for April
			int[,] yA5 = new int[33, 1001];	// Year array for May
			string sp;
			Common.WriteDLn("Gregorian Calendar");
			Common.WriteLn(" Date   Years when Orthodox Easter falls on a given date for years " + startYear + " - " + endYear);
			for (day = 1; day <= 31; day++)
			{
				tA3[day] = 0;
				tA4[day] = 0;
				tA5[day] = 0;
			}
			times = 0;
			for (year = startYear; year <= endYear; year++)
			{
				EasterO(out oJDay, out oJMonth, out oGDay, out oGMonth, year);
				if (oGMonth == 3)
				{
					tA3[oGDay]++;
					yA3[oGDay, tA3[oGDay]] = year;
					times++;
				}
				else if (oGMonth == 4)
				{
					tA4[oGDay]++;
					yA4[oGDay, tA4[oGDay]] = year;
					times++;
				}
				else
				{
					if (oGDay > 32)
						oGDay = 32;
					if (tA5[oGDay] == 1000)
						yA5[oGDay, tA5[oGDay]] = 0;
					if (tA5[oGDay] < 1000)
					{
						tA5[oGDay]++;
						yA5[oGDay, tA5[oGDay]] = year;
						times++;
					}
				}
			}
			int startM = 0;
			int startD = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] != 0)
				{
					startD = day;
					startM = 3;
					break;
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 30; day++)
				{
					if (tA4[day] != 0)
					{
						startD = day;
						startM = 4;
						break;
					}
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 32; day++)
				{
					if (tA5[day] != 0)
					{
						startD = day;
						startM = 5;
						break;
					}
				}
			}
			startD--;
			if (startD < 1)
			{
				startM--;
				startD = 31;
				if (startM == 4)
					startD = 30;
			}
			int stopM = 0;
			int stopD = 0;
			for (day = 32; day >= 1; day--)
			{
				if (tA5[day] != 0)
				{
					stopD = day;
					stopM = 5;
					break;
				}
			}
			if (stopM == 0)
			{
				for (day = 30; day >= 1; day--)
				{
					if (tA4[day] != 0)
					{
						stopD = day;
						stopM = 4;
						break;
					}
				}
			}
			if (stopM == 0)
			{
				for (day = 31; day >= 1; day--)
				{
					if (tA3[day] != 0)
					{
						stopD = day;
						stopM = 3;
						break;
					}
				}
			}
			stopD++;
			if (stopD > 31 || (stopD > 30 && stopM == 4))
			{
				if (stopM == 5)
					stopD = 32;
				else
				{
					stopM++;
					stopD = 1;
				}
			}
			int start;
			int stop;
			if (startM == 3)
			{
				stop = 31;
				if (stopM == 3)
					stop = stopD;
				for (day = startD; day <= stop; day++)
				{
					Common.Write(" 3/" + day + "  ");
					if (tA3[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA3[day]; i++)
					{
						Common.Write(" " + yA3[day, i].ToString("0000"));
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
					}
					Common.WriteLn();
					if (Common.abortCalc)
						break;
				}
			}
			if (startM <= 4 && stopM >= 4)
			{
				start = 1;
				stop = 30;
				if (startM == 4)
					start = startD;
				if (stopM == 4)
					stop = stopD;
				for (day = start; day <= stop; day++)
				{
					if (Common.abortCalc)
						break;
					sp = "";
					if (day < 10) sp = " ";
					Common.Write(" 4/" + sp + day + "  ");
					if (tA4[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA4[day]; i++)
					{
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
						Common.Write(" " + yA4[day, i].ToString("0000"));
					}
					Common.WriteLn();
				}
			}
			if (stopM == 5)
			{
				start = 1;
				if (startM == 5)
					start = startD;
				for (day = start; day <= stopD; day++)
				{
					if (Common.abortCalc)
						break;
					if (day > 31)
						Common.Write(" >May  ");
					else
					{
						sp = "";
						if (day < 10) sp = " ";
						Common.Write(" 5/" + sp + day + "  ");
					}
					if (tA5[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA5[day]; i++)
					{
						if (yA5[day, i] == 0)
							Common.Write(" ... etc.");
						else
							Common.Write(" " + yA5[day, i].ToString("0000"));
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
					}
					Common.WriteLn();
				}
			}
			if (Common.abortCalc)
				Common.WriteLn("Manually aborted before list finished");
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		} // OByGDate

		public static void OByGFreq()	// like EasterG.QB
		// Frequency of Dates of Orthodox Easter, given in the Gregorian Calendar, StartYear - EndYear
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int day;					// Day tabulating
			int month;					// Month tabulating
			int oGDay;					// Day, 1 through 31 Orthodox Gregorian date, May 32 for overflow
			int oGMonth;				// Month, 3, 4 or 5 Orthodox Gregorian date
			int oJDay;					// Day, 1 through 31 Orthodox Julian date
			int oJMonth;				// Month, 3 or 4 Orthodox Julian date
			int span;					// NJumber of years to include
			int times;					// Total times tabulated
			int year;					// Year to compute for
			int[] tA3 = new int[32];	// Times array for March
			int[] tA4 = new int[32];	// Times array for April
			int[] tA5 = new int[33];	// Times array for May
			Single p;					// Percent
			span = endYear - startYear + 1;
			Common.WriteDLn("Gregorian Calendar, Frequency of Dates of Orthodox Easter " + startYear + " - " + endYear);
			for (day = 1; day <= 31; day++)
			{
				tA3[day] = 0;
				tA4[day] = 0;
				tA5[day] = 0;
			}
			times = 0;
			for (year = startYear; year <= endYear; year++)
			{
				EasterO(out oJDay, out oJMonth, out oGDay, out oGMonth, year);
				if (oGMonth == 3)
				{
					tA3[oGDay]++;
					times++;
				}
				else if (oGMonth == 4)
				{
					tA4[oGDay]++;
					times++;
				}
				else
				{
					if (oGDay > 32)
						oGDay = 32;
					tA5[oGDay]++;
					times++;
				}
			}
			int startM = 0;
			int startD = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] != 0)
				{
					startD = day;
					startM = 3;
					break;
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 30; day++)
				{
					if (tA4[day] != 0)
					{
						startD = day;
						startM = 4;
						break;
					}
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 32; day++)
				{
					if (tA5[day] != 0)
					{
						startD = day;
						startM = 5;
						break;
					}
				}
			}
			startD--;
			if (startD < 1)
			{
				startM--;
				startD = 31;
				if (startM == 4)
					startD = 30;
			}
			int stopM = 0;
			int stopD = 0;
			for (day = 32; day >= 1; day--)
			{
				if (tA5[day] != 0)
				{
					stopD = day;
					stopM = 5;
					break;
				}
			}
			if (stopM == 0)
			{
				for (day = 30; day >= 1; day--)
				{
					if (tA4[day] != 0)
					{
						stopD = day;
						stopM = 4;
						break;
					}
				}
			}
			if (stopM == 0)
			{
				for (day = 31; day >= 1; day--)
				{
					if (tA3[day] != 0)
					{
						stopD = day;
						stopM = 3;
						break;
					}
				}
			}
			stopD++;
			if (stopD > 31 || (stopD > 30 && stopM == 4))
			{
				if (stopM == 5)
					stopD = 32;
				else
				{
					stopM++;
					stopD = 1;
				}
			}
			int maxTimes = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] > maxTimes)
					maxTimes = tA3[day];
				if (tA4[day] > maxTimes)
					maxTimes = tA4[day];
				if (tA5[day] > maxTimes)
					maxTimes = tA5[day];
			}
			string dayTimes = maxTimes.ToString();
			int length = dayTimes.Length;
			int start;
			int stop;
			month = 3;
			if (startM == 3)
			{
				stop = 31;
				if (stopM == 3)
					stop = stopD;
				for (day = startD; day <= stop; day++)
				{
					p = 100f * tA3[day] / span;
					dayTimes = Format(tA3[day], length);
					Common.Write("Easter Sunday: " + month.ToString() + "/" + day + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + " %");
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
				}
			}
			month = 4;
			if (startM <= 4 && stopM >= 4)
			{
				start = 1;
				stop = 30;
				if (startM == 4)
					start = startD;
				if (stopM == 4)
					stop = stopD;
				for (day = start; day <= stop; day++)
				{
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
					p = 100f * tA4[day] / span;
					dayTimes = Format(tA4[day], length);
					Common.Write("Easter Sunday: " + month.ToString() + "/" + Format(day, 2) + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + " %");
				}
			}
			month = 5;
			if (stopM == 5)
			{
				start = 1;
				if (startM == 5)
					start = startD;
				for (day = start; day <= stopD; day++)
				{
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
					p = 100f * tA5[day] / span;
					dayTimes = Format(tA5[day], length);
					string sMD = month.ToString() + "/" + Format(day, 2);
					if (day > 31)
						sMD = ">May";
					Common.Write("Easter Sunday: " + sMD + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + " %");
				}
			}
			if (Common.abortCalc)
				Common.WriteLn("Manually aborted before list finished");
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		} // OByGFreq

		public static void OByJDate()	// like EasterH.QB
		// Dates of Orthodox Easter, ordered by date, given in the Julian Calendar, StartYear - EndYear
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int day;							// Day tabulating
			int i;								// Index
			int oGDay;							// Day, 1 through 31 Orthodox Gregorian date
			int oGMonth;						// Month, 3, 4 or 5 Orthodox Gregorian date
			int oJDay;							// Day, 1 through 31 Orthodox Julian date
			int oJMonth;						// Month, 3 or 4 Orthodox Julian date
			int times;							// Total times tabulated
			int year;							// Year to compute for
			int[] tA3 = new int[100];			// Times array for March
			int[] tA4 = new int[100];			// Times array for April
			int[,] yA3 = new int[32, 376000];	// Year array for March
			int[,] yA4 = new int[26, 376000];	// Year array for April
			string sp;
			Common.WriteDLn("Julian Calendar");
			Common.WriteLn(" Date   Years when Orthodox Easter falls on a given date for years " + startYear + " - " + endYear);
			for (day = 1; day <= 31; day++)
			{
				tA3[day] = 0;
				tA4[day] = 0;
			}
			times = 0;
			for (year = startYear; year <= endYear; year++)
			{
				EasterO(out oJDay, out oJMonth, out oGDay, out oGMonth, year);
				if (oJMonth == 3)
				{
					tA3[oJDay]++;
					yA3[oJDay, tA3[oJDay]] = year;
				}
				else
				{
					tA4[oJDay]++;
					yA4[oJDay, tA4[oJDay]] = year;
				}
				times++;
			}
			int startM = 0;
			int startD = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] != 0)
				{
					startD = day;
					startM = 3;
					break;
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 30; day++)
				{
					if (tA4[day] != 0)
					{
						startD = day;
						startM = 4;
						break;
					}
				}
			}
			startD--;
			if (startD < 1)
			{
				startM--;
				startD = 31;
			}
			int stopM = 0;
			int stopD = 0;
			for (day = 30; day >= 1; day--)
			{
				if (tA4[day] != 0)
				{
					stopD = day;
					stopM = 4;
					break;
				}
			}
			if (stopM == 0)
			{
				for (day = 31; day >= 1; day--)
				{
					if (tA3[day] != 0)
					{
						stopD = day;
						stopM = 3;
						break;
					}
				}
			}
			stopD++;
			if (stopD > 31)
			{
				stopM++;
				stopD = 1;
			}
			int start;
			int stop;
			if (startM == 3)
			{
				stop = 31;
				if (stopM == 3)
					stop = stopD;
				for (day = startD; day <= stop; day++)
				{
					Common.Write(" 3/" + day + "  ");
					if (tA3[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA3[day]; i++)
					{
						Common.Write(" " + yA3[day, i].ToString("0000"));
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
					}
					Common.WriteLn();
					if (Common.abortCalc)
						break;
				}
			}
			if (stopM == 4)
			{
				start = 1;
				if (startM == 4)
					start = startD;
				for (day = start; day <= stopD; day++)
				{
					if (Common.abortCalc)
						break;
					sp = "";
					if (day < 10) sp = " ";
					Common.Write(" 4/" + sp + day + "  ");
					if (tA4[day] == 0) Common.Write(" none");
					for (i = 1; i <= tA4[day]; i++)
					{
						Common.MuInterrupt();
						if (Common.abortCalc)
							break;
						Common.Write(" " + yA4[day, i].ToString("0000"));
					}
					Common.WriteLn();
				}
			}
			if (Common.abortCalc)
				Common.WriteLn("Manually aborted before list finished");
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		} // OByJDate

		public static void OByJFreq()	// like EasterH.QB
		// Frequency of Dates of Orthodox Easter, given in the Julian Calendar, StartYear - EndYear
		{
			if (Common.running) return;
			Global.objRunForm.SetRunning();
			Common.abortCalc = false;
			int day;					// Day tabulating
			int month;					// Month tabulating
			int oGDay;					// Day, 1 through 31 Orthodox Gregorian date
			int oGMonth;				// Month, 3, 4 or 5 Orthodox Gregorian date
			int oJDay;					// Day, 1 through 31 Orthodox Julian date
			int oJMonth;				// Month, 3 or 4 Orthodox Julian date
			int span;					// NJumber of years to include
			int times;					// Total times tabulated
			int year;					// Year to compute for
			Single p;					// Percent
			span = endYear - startYear + 1;
			label = "Julian Calendar, Frequency of Dates of Orthodox Easter " + startYear + " - " + endYear;
			Common.WriteDLn(label);
			for (day = 1; day <= 31; day++)
			{
				tA3[day] = 0;
				tA4[day] = 0;
			}
			times = 0;
			for (year = startYear; year <= endYear; year++)
			{
				EasterO(out oJDay, out oJMonth, out oGDay, out oGMonth, year);
				if (oJMonth == 3)
				{
					tA3[oJDay]++;
				}
				else
				{
					tA4[oJDay]++;
				}
				times++;
			}
			int startM = 0;
			int startD = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] != 0)
				{
					startD = day;
					startM = 3;
					break;
				}
			}
			if (startM == 0)
			{
				for (day = 1; day <= 30; day++)
				{
					if (tA4[day] != 0)
					{
						startD = day;
						startM = 4;
						break;
					}
				}
			}
			startD--;
			if (startD < 1)
			{
				startM--;
				startD = 31;
			}
			int stopM = 0;
			int stopD = 0;
			for (day = 30; day >= 1; day--)
			{
				if (tA4[day] != 0)
				{
					stopD = day;
					stopM = 4;
					break;
				}
			}
			if (stopM == 0)
			{
				for (day = 31; day >= 1; day--)
				{
					if (tA3[day] != 0)
					{
						stopD = day;
						stopM = 3;
						break;
					}
				}
			}
			stopD++;
			if (stopD > 31)
			{
				stopM++;
				stopD = 1;
			}
			int maxTimes = 0;
			for (day = 1; day <= 31; day++)
			{
				if (tA3[day] > maxTimes)
					maxTimes = tA3[day];
				if (tA4[day] > maxTimes)
					maxTimes = tA4[day];
			}
			string dayTimes = maxTimes.ToString();
			int length = dayTimes.Length;
			int start;
			int stop;
			month = 3;
			if (startM == 3)
			{
				stop = 31;
				if (stopM == 3)
					stop = stopD;
				for (day = startD; day <= stop; day++)
				{
					p = 100f * tA3[day] / span;
					dayTimes = Format(tA3[day], length);
					Common.Write("Easter Sunday: " + month.ToString() + "/" + day + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + "%");
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
				}
			}
			month = 4;
			if (stopM == 4)
			{
				start = 1;
				if (startM == 4)
					start = startD;
				for (day = start; day <= stopD; day++)
				{
					Common.MuInterrupt();
					if (Common.abortCalc)
						break;
					p = 100f * tA4[day] / span;
					dayTimes = Format(tA4[day], length);
					Common.Write("Easter Sunday: " + month.ToString() + "/" + Format(day, 2) + "  " + dayTimes);
					Common.WriteLn(" Times in " + span + " Years, " + p + "%");
				}
			}
			if (Common.abortCalc)
			{
				Common.WriteLn("Manually aborted before list finished");
				label = "";
			}
			Common.WriteLn2("Total years tabulated = " + times);
			Common.abortCalc = false;
			Global.objRunForm.SetNotRunning();
		} // OByJFreq

		public static string Format(int i, int n)	// format integer in n spaces
		{
			string st;
			st = i.ToString();
			for (int j = st.Length; j < n; j++)
				st = " " + st;
			return st;
		} // Format

		public static string GetField(string st)	// get the next numeric field from st
		{
			string f = "";
			if (st.Length == 0)
				return f;
			f += st[0];			// first character can be - or + or a digit
			for (int i = 1; i < st.Length; i++)
			{
				if (st[i] >= '0' && st[i] <= '9')
					f += st[i];
				else
					break;
			}
			return f;
		} // GetField

		public static void FixCurrYear()	// put currYear in range
		{
			if (currYear > maxYear)
				currYear = maxYear;
			else if (currYear < 1)
				currYear = 1;
		} // FixCurrYear

		public static void Run(StreamReader inCF)	// Run a code file
		{
			int i;
			bool done = false;
			string st;
			Common.muAbort = false;
			Common.WriteLn();
			do
			{
				st = inCF.ReadLine();		// read next command from file
				if (st == null) break;		// null => EOF (end of file)
				Common.WriteTLn(" " + st);			// echo to output textbox
				st = st.Trim(' ');
				string stO = st;
				st = st.ToUpper();
				if (st.Length >= 2 && st.Substring(0, 2) == "//")
					st = "//";
				else if (st.Length >= 4 && st.Substring(0, 4) == "YEAR")
				{
					st = st.Substring(4);
					st = st.TrimStart(' ');
					if (st.Length >= 1 && st.Substring(0, 1) == "=")
					{
						st = st.TrimStart('=', ' ');
						st = GetField(st);
						if (Global.IsInt32(st))
						{
							EasterCS.currYear = Convert.ToInt32(st.Substring(0));
							EasterCS.FixCurrYear();
							EasterCS.DisplayOneYear(EasterCS.currYear);
						}
					}
					else if (st.Length >= 1 && st.Substring(0, 1) == "+")
					{
						EasterCS.currYear++;
						EasterCS.FixCurrYear();
						EasterCS.DisplayOneYear(EasterCS.currYear);
					}
					else if (st.Length >= 1 && st.Substring(0, 1) == "-")
					{
						EasterCS.currYear--;
						EasterCS.FixCurrYear();
						EasterCS.DisplayOneYear(EasterCS.currYear);
					}
				}
				else if (st.Length >= 9 && st.Substring(0, 9) == "STARTYEAR")
				{
					st = st.Substring(9);
					st = st.TrimStart('=', ' ');
					st = GetField(st);
					if (Global.IsInt32(st))
						EasterCS.startYear = Convert.ToInt32(st.Substring(0));
				}
				else if (st.Length >= 7 && st.Substring(0, 7) == "ENDYEAR")
				{
					st = st.Substring(7);
					st = st.TrimStart('=', ' ');
					st = GetField(st);
					if (Global.IsInt32(st))
						EasterCS.endYear = Convert.ToInt32(st.Substring(0));
				}
				else if (st.Length >= 8 && st.Substring(0, 8) == "WOBYYEAR")
				{
					EasterCS.WOByYear();
				}
				else if (st.Length >= 7 && st.Substring(0, 7) == "WBYDATE")
				{
					EasterCS.WByDate();
				}
				else if (st.Length >= 5 && st.Substring(0, 5) == "WFREQ")
				{
					EasterCS.WFreq();
				}
				else if (st.Length >= 8 && st.Substring(0, 8) == "OBYGFREQ")
				{
					EasterCS.OByGFreq();
				}
				else if (st.Length >= 8 && st.Substring(0, 8) == "OBYJFREQ")
				{
					EasterCS.OByJFreq();
				}
				else if (st.Length >= 8 && st.Substring(0, 8) == "OBYGDATE")
				{
					EasterCS.OByGDate();
				}
				else if (st.Length >= 8 && st.Substring(0, 8) == "OBYJDATE")
				{
					EasterCS.OByJDate();
				}
				else if (st.Length >= 9 && st.Substring(0, 9) == "LOGSCREEN")
				{
					st = st.Substring(9);
					st = st.TrimStart(' ', '(');
					st = st.Substring(0, 1);
					if (st == "0" && Common.logg) Common.ToggleLogg();
					else if (st != "0" && !Common.logg) Common.ToggleLogg();
				}
				else if (st.Length >= 8 && st.Substring(0, 8) == "CLEARLOG")
				{
					Common.ClearLog();
					Global.objRunForm.SetLogCaption();
				}
				else if (st.Length >= 6 && st.Substring(0, 6) == "LOGOUT")
				{
					st = stO.Substring(6);
					st = st.TrimStart(' ', '(', '"');
					i = st.IndexOf("\"", 0);
					if (i >= 0)
						st = st.Substring(0, i);
					i = st.IndexOf(" ", 0);
					if (i >= 0)
						st = st.Substring(0, i);
					Common.FixFName(ref st, "log");
					Common.logFileNew = st;
				}
				else if (st.Length >= 4 && st.Substring(0, 4) == "EXIT")
				{
					Global.objRunForm.Exit();
					return;
				}
			}
			while (!done && !Common.muAbort);
			Common.WriteLn();
		} // Run

		public static int currYear;		// current year to compute Easter date for
		public static int endYear;		// end of range of years
		public static int maxYear;		// Max Year allowed
		public static int startYear;	// start of range of years
		public static int[] tA3 = new int[32];	// Times array for March WFreq
		public static int[] tA4 = new int[32];	// Times array for April WFreq
		public static string label;			// label for plot form
}
}
